import 'package:flutter/material.dart';
import 'package:js_ui_components/js_ui_components.dart';

enum JsButtonType { text, outlined, elevated }

class JsButton extends StatelessWidget {
  final ButtonStyle style;
  final JsButtonType type;
  final Widget child;
  final String text;
  final Function onPressed;
  final Function onLongPress;
  final EdgeInsets padding;
  final bool enabled;
  final bool loading;

  JsButton({
    this.type = JsButtonType.elevated,
    this.child,
    this.onPressed,
    this.padding,
    this.text,
    this.enabled = true,
    this.loading,
    this.style,
    this.onLongPress,
  });

  @override
  Widget build(BuildContext context) {
    if (type == JsButtonType.outlined)
      return OutlinedButton(
        onPressed: (enabled == null || enabled == false) ? null : onPressed,
        onLongPress: (enabled == null || enabled == false) ? null : onLongPress,
        child: _getChild(),
        style: style ?? Theme.of(context).outlinedButtonTheme.style,
      );

    if (type == JsButtonType.text)
      return TextButton(
        onPressed: (enabled == null || enabled == false) ? null : onPressed,
        onLongPress: (enabled == null || enabled == false) ? null : onLongPress,
        child: _getChild(),
        style: style ?? Theme.of(context).textButtonTheme.style,
      );

    return ElevatedButton(
      onPressed: (enabled == null || enabled == false) ? null : onPressed,
      onLongPress: (enabled == null || enabled == false) ? null : onLongPress,
      child: _getChild(),
      style: style ?? Theme.of(context).elevatedButtonTheme.style,
    );
  }

  Widget _getChild() {
    if (loading != null && loading)
      return Container(
        alignment: Alignment.center,
        child: JsProgress(),
      );

    if (child != null) return child;

    if (text != null)
      return Container(
        alignment: Alignment.center,
        child: Text(text),
      );

    return null;
  }
}
